# GIE-CLF
Official Implementation of Learning Control Lyapunov Functions For High-dimensional Unknown Systems using Guided Iterative State Space Exploration

## Dependencies
To install requirements:

```bash
conda create -n gie_clf python=3.8
conda activate gie_clf
pip install -r requirements.txt
```

To run the F-16 environments, you need to install the [AeroBenchVVPython](https://github.com/stanleybak/AeroBenchVVPython) package. To install this package, use 

```bash
git clone git@github.com:stanleybak/AeroBenchVVPython.git
```

Then you need to change the 8th line in the file `model/env/aerobench.py`. Change `CUSTOMER_DIR` to the dir of the AeroBenchVVPython package on your computer. 

## Environments
We provide 4 environments, including InvertedPendulum, NeuralLander, F16GCA, and F16Tracking.

## Run
### Hyper-parameters
To reproduce the results shown in our paper, you can use the hyper-parameters we provided in `./hyperparam.yaml`.

### Collect Demonstrations
Use the following command to collect demonstrations:

```bash
python collect_demo.py --env InvertedPendulum --std 0.1 --bias 4.0
```

Here `--std` specifies the standard deviation of the gaussian noise added to the action, and `--bias` specifies the bias added to the action. The parameters we used in our experiments are:

|Environment|std|bias|
|-----------|---|----|
|InvertedPendulum|0.1|4.0|
|NeuralLander|0.1|0.1|
|F16GCA|0.005|0.01|
|F16Tracking|0.1|0.1|

The collected demonstrations will show up in folder `demos/<ENV>`.

### Train GIE-CLF
After collecting demonstrations, we are ready to train the GIE-CLF to get the controller. Use the following command to train a GIE-CLF:

```bash
python train_clf.py --env InvertedPendulum --buffer ./demos/InvertedPendulum/size20000_std0.1_bias4.0_reward1594.pkl --iter-outer 20 --use-baseline 10
```

### Train RL or IL Agents
We provide implementations of PPO, AIRL, D-REX, and SSRR as our baselines. Use the following line to train these algorithms:

```bash
python train_rl_il.py --env InvertedPendulum --buffer ./demos/InvertedPendulum/size20000_std0.1_bias4.0_reward1594.pkl --algo ppo --steps 2000000
```

Note that if you want to train an SSRR agent, you need to train an AIRL agent first, and then use `--airl-path` to specify the path to the trained AIRL agent for SSRR.

### Evaluation
Once an algorithm is well-trained, you can use the following command to evaluate it:

```bash
python test_policy.py --path ./logs/InvertedPendulum/model/seed0_<TRAINING_TIME>
```